package ro.sync.util.editorvars.parser.model;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * An $ask editor variable. 
 * <pre>${ask('message', combobox, ('real_value1':'rendered_value1';...;'real_valueN':'rendered_valueN'), 'default')}</pre>
 * 
 * @author alex_jitianu
 * @author sorin_carbunaru
 */
public class AskEditorVariable extends EditorVariable {
  /**
   * Editor variable name.
   */
  private static final String ASK_NAME = "ask";

  /**
   * The message to present to the user.
   */
  private LexicalItem message;
  /**
   * What type of input we request from the user: combobox, radio, etc.
   */
  private LexicalItem inputType;
  /**
   * value=>label mappings.
   */
  private Map<LexicalItem, LexicalItem> valueEnumeration;
  /**
   * The value enumeration part was specified as a lexical item, most likely as another editor variable.
   * When this happens, we need to:
   * <ol>
   * <li>Expand/resolve the lexical unit and obtain the string value</li>
   * <li>Serialize the entire $ask editor variable to its string representation.</li>
   * <li>Parse the $ask again so that we now have proper value:label pairs.</li> 
   * </ol>
   * <pre>
   * ${ask('combobox with quotes', combobox, ( ${xpath_eval('real_value1':'rendered_value1')} ))}
   * </pre>
   */
  private LexicalItem pendingValueEnumeration;
  /**
   * Default value.
   */
  private LexicalItem defaultValue;
  /**
   * The value obtained from the user will be linked to this ID. 
   * Later on, this ID can be used with the $answer editor variable.
   */
  private LexicalItem answerId;

  /**
   * Constructor.
   * 
   * @param message The message to present to the user.
   * @param inputType What type of input we request from the user: combobox, radio, etc.
   * @param valueEnumeration value=>label mappings.
   * @param defaultValue Default value.
   * @param answerId The value obtained from the user will be linked to this ID. 
   * Later on, this ID can be used with the $answer editor variable.
   */
  public AskEditorVariable(
      LexicalItem message,
      LexicalItem inputType,
      Map<LexicalItem, LexicalItem> valueEnumeration, 
      LexicalItem defaultValue, 
      LexicalItem answerId) {
    super(ASK_NAME, Type.ASK_EDITOR_VARIABLE);
    
    this.message = message;
    this.inputType = inputType;
    this.valueEnumeration = valueEnumeration;
    if (this.valueEnumeration == null) {
      this.valueEnumeration = new HashMap<>();
    }
    this.defaultValue = defaultValue;
    this.answerId = answerId;
  }
  
  /**
   * Constructor.
   * 
   * @param message The message to present to the user.
   * @param inputType What type of input we request from the user: combobox, radio, etc.
   * @param pendingValueEnumeration An editor variable that will be expanded to the actual value:label mapping..
   * @param defaultValue Default value.
   * @param answerId The value obtained from the user will be linked to this ID. 
   * Later on, this ID can be used with the $answer editor variable.
   */
  public AskEditorVariable(
      LexicalItem message,
      LexicalItem inputType,
      LexicalItem pendingValueEnumeration, 
      LexicalItem defaultValue, 
      LexicalItem answerId) {
    super(ASK_NAME, Type.ASK_EDITOR_VARIABLE);
    
    this.message = message;
    this.inputType = inputType;
    this.pendingValueEnumeration = pendingValueEnumeration;
    this.defaultValue = defaultValue;
    this.answerId = answerId;
  }
  
  /**
   * @return The message to present to the user.
   */
  public LexicalItem getMessage() {
    return message;
  }

  /**
   * @return What type of input we request from the user: combobox, radio, etc.
   */
  public LexicalItem getInputType() {
    return inputType;
  }

  /**
   * @return value=>label mappings.
   */
  public Map<LexicalItem, LexicalItem> getValueEnumeration() {
    return valueEnumeration;
  }
  
  /**
   * @return <code>true</code> if the value:label map is not computed yet.
   * 
   * @see #getPendingValueEnumeration()
   */
  public boolean hasPendingValue() {
    return pendingValueEnumeration != null;
  }
  
  /**
   * The value enumeration part was specified as a lexical item, most likely as another editor variable.
   * When this happens, we need to:
   * <ol>
   * <li>Expand/resolve the lexical unit and obtain the string value</li>
   * <li>Serialize the entire $ask editor variable to its string representation.</li>
   * <li>Parse the $ask again so that we now have proper value:label pairs.</li> 
   * </ol>
   * <pre>
   * ${ask('combobox with quotes', combobox, ( ${xpath_eval('real_value1':'rendered_value1')} ))}
   * </pre>
   * 
   * @return The value enumeration part was specified as a lexical item, most likely as another editor variable.
   */
  public LexicalItem getPendingValueEnumeration() {
    return pendingValueEnumeration;
  }

  /**
   * @return Default value.
   */
  public LexicalItem getDefaultValue() {
    return defaultValue;
  }

  /**
   * @return The value obtained from the user will be linked to this ID. 
   * Later on, this ID can be used with the $answer editor variable.
   */
  public LexicalItem getAnswerId() {
    return answerId;
  }
  
  @Override
  public List<LexicalItem> getParams() {
    List<LexicalItem> params = new LinkedList<>();
    
    if (message != null) {
      params.add(message);
    }
    
    if (inputType != null) {
      params.add(inputType);
    }
    
    if (valueEnumeration != null) {
      Set<LexicalItem> keySet = valueEnumeration.keySet();
      for (LexicalItem lexicalItem : keySet) {
        params.add(lexicalItem);
        LexicalItem label = valueEnumeration.get(lexicalItem);
        if (label != null) {
          params.add(label);
        }
      }
    } else if (pendingValueEnumeration != null) {
      params.add(pendingValueEnumeration);
    }
    
    if (defaultValue != null) {
      params.add(defaultValue);
    }
    
    if (answerId != null) {
      params.add(answerId);
    }
    
    return params;
  }
  
  @Override
  public void setParams(List<LexicalItem> params) {
    throw new IllegalStateException("Unimplemented");
  }
}
